library wasmcamera;

{$mode objfpc}
{$h+}
{$codepage UTF8}

uses
  SysUtils, Variants, Job.Js, JOB_Web;

type
  TCameraApp = class
  private
    StillButton: IJSHTMLButtonElement;
    StartButton: IJSHTMLButtonElement;
    Video: IJSHTMLVideoElement;
    Canvas: IJSHTMLCanvasElement;
    Context: IJSCanvasRenderingContext2D;
    procedure StartVideo(Event: IJSEvent);
    procedure CreateStill(Event: IJSEvent);
    function StartStream(const Res : Variant) : Variant;
    function DoError(const Res : Variant) : Variant;
  public
    procedure Run;
  public

  end;

{ TCameraApp }

procedure TCameraApp.CreateStill(Event: IJSEvent);

begin
  Canvas.width:=Video.clientWidth;
  Canvas.height:=Video.clientHeight;
  Context.drawImage(Video,0,0,Video.ClientWidth,Video.ClientHeight);
end;

procedure TCameraApp.Run;
begin
  Video := TJSHTMLVideoElement.Cast(JSDocument.getElementById('video'));
  Canvas := TJSHTMLCanvasElement.Cast(JSDocument.getElementById('canvas'));
  Context := TJSCanvasRenderingContext2D.Cast(Canvas.getContext('2d'));

  StartButton:=TJSHTMLButtonElement.Cast(JSDocument.getElementById('start'));
  StartButton.InnerHTML := 'Start video';
  StartButton.addEventListener('click', @StartVideo);

  StillButton := TJSHTMLButtonElement.Cast(JSDocument.getElementById('still'));
  StillButton.InnerHTML := 'Create still';
  StillButton.addEventListener('click', @CreateStill);
end;

Procedure TCameraApp.StartVideo(Event: IJSEvent);

var
  constraints : TJSMediaStreamConstraints;

begin
  constraints:=TJSMediaStreamConstraints.Create;
  constraints.video:=True;
  constraints.audio:=False;
  JSWindow.navigator.mediaDevices.getUserMedia(Constraints)
    ._then(@StartStream)
    .catch(@DoError)
end;

function TCameraApp.StartStream(const Res : Variant) : Variant; stdcall;

var
  Stream : IJSMediaStream;

begin
  Stream:=IJSMediaStream(Res);
  Video := TJSHTMLVideoElement.Cast(JSDocument.getElementById('video'));
  Video.srcObject := Stream;
  Video.play();
end;

function TCameraApp.DoError(const Res : Variant) : Variant; stdcall;

begin
  writeln('Error accessing the webcam: '+string(Res));
end;

exports
  JOBCallback;

begin
  With TCameraApp.Create do
    Run;
end.
